#include "uart.h"
#include "ddr_sdram.h"
//#include "sdram_detect.h"
#include "mem_test.h"
#include "xparameters.h"

#define SDRAM_DATA_WIDTH 64
#define SDRAM_ADDR_WIDTH 28
#define SDRAM_WORD_COUNT 67108864 - 0x5000 //67108864
#define SDRAM_MAX_OFFSET 0x0FFFFFFF

static Xuint32 DDR_BaseAddress;

// funtion protos
int runTestPatternRGB(int  word_count);
void sdramInit(Xuint32 sdram_address) {
  DDR_BaseAddress = sdram_address;
} // end sdramInit()


int main() {
  Xuint32 data_width = SDRAM_DATA_WIDTH;
  Xuint32 addr_width = SDRAM_ADDR_WIDTH;
  Xuint32 word_count = SDRAM_WORD_COUNT;
  Xuint32 max_offset = SDRAM_MAX_OFFSET;
  Xuint32 base_address, i;
  Xuint32 rank_count;

  print("\r\nTurning caches on");
  XCache_EnableDCache(0xF000000F); 
  XCache_EnableICache(0xF000000F);
   
  sdramInit(XPAR_DDR_512MB_64MX64_RANK1_ROW13_COL11_CL2_5_MEM0_BASEADDR);
  DDR_BaseAddress = DDR_BaseAddress + 0x5000;

  //clearScreen();

  print("\r\nSDRAM Module Detection");
  print("\r\n----------------------\r\n");
  print("Checking available DIMM module...\t");
  //rank_count = sdramDetect();
  rank_count = 1;
  if( rank_count <= 0 ) { 
    
    return;
  }
  print("done!\r\n");
  xil_printf("Ranks detected: %d\r\n", rank_count);

  for( i = 0; i < rank_count; i++ ) {
    base_address = DDR_BaseAddress + (i * (0x10000000 - 0x5000));
    xil_printf("\r\nDDR SDRAM Test: Rank %d", i);
    print("\r\n----------------------\r\n");
    runDataTestL(base_address, data_width);
    runAddrTest(base_address, addr_width, max_offset);
    runDeviceTestL(base_address, word_count);
  }

  // write color bars test pattern to video memory area
  runTestPatternRGB(655360);

  print("\r\nMemory test complete\r\n");
  

} // end sdramTest()


int runTestPatternRGB(int  word_count) {
  unsigned int i, x, wordnum;
  volatile unsigned int rgb_value;
  unsigned int block_start; 
  unsigned int pattern;
  unsigned int test_val;
  unsigned int  *pDisplay_data;

  pDisplay_data   = (unsigned int  *)0x07E00000;

  print("\n\rRunning rgb Test Pattern...\t");

  print("...\t");
  x = wordnum = 0;

  // write pattern to DDR
  while (wordnum < word_count)
	{
		// Color pattern
		if (x < 80)
			rgb_value = 0x00FFFFFF; // white
		else if (x < 160)
			rgb_value = 0x00000000; // black
		else if (x < 240)
			rgb_value = 0x00FFFF00; // yellow
		else if (x < 320)
			rgb_value = 0x0000FFFF; // cyan
		else if (x < 400)
			rgb_value = 0x0000FF00; // green
		else if (x < 480)
			rgb_value = 0x00FF00FF; // magenta
		else if (x < 560)
			rgb_value = 0x00FF0000; // red
		else if (x < 640)
			rgb_value = 0x000000FF; // blue

		*(pDisplay_data) = rgb_value;
		pDisplay_data++;
		wordnum++;
		x++;
		if (x >= 640){
		 x = 0;
		 pDisplay_data += (1024 - 640);
		 }

  } // while wordnum < word_count


  print("Passed!\n\r");
  return ;

} // end runTestPatternrgb
